/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkRemoveGhostCells.h,v $
  Language:  C++
  Date:      $Date: 2000/12/10 20:08:50 $
  Version:   $Revision: 1.3 $


Copyright (c) 1993-2001 Ken Martin, Will Schroeder, Bill Lorensen 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

 * Neither name of Ken Martin, Will Schroeder, or Bill Lorensen nor the names
   of any contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 * Modified source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/
// .NAME vtkRemoveGhostCells - Removes ghost cells whose ghost level is above
// a specified value

#ifndef __vtkRemoveGhostCells_h
#define __vtkRemoveGhostCells_h

#include "vtkPolyDataToPolyDataFilter.h"

class VTK_EXPORT vtkRemoveGhostCells : public vtkPolyDataToPolyDataFilter
{
public:
  static vtkRemoveGhostCells *New();
  vtkTypeMacro(vtkRemoveGhostCells, vtkPolyDataToPolyDataFilter);
  void PrintSelf(ostream& os, vtkIndent indent);
  
  // Description:
  // Set / Get the lowest ghost level to remove.  (All cells with ghost
  // levels at this level or higher will be removed.)
  vtkSetMacro(GhostLevel, int);
  vtkGetMacro(GhostLevel, int);

protected:
  vtkRemoveGhostCells();
  ~vtkRemoveGhostCells() {};
  vtkRemoveGhostCells(const vtkRemoveGhostCells&) {};
  void operator=(const vtkRemoveGhostCells&) {};

  // Usual data generation method
  void Execute();

  int GhostLevel;
};

#endif
